unit rkGlassButton;

{$WARN SYMBOL_PLATFORM OFF}
{$WARN UNIT_PLATFORM OFF}

//  GlassButton by Roy Magne Klever
//   2010 by Roy Magne Klever. All rights reserved
//
//  This file is not distributable without permission by Roy Magne Klever
//  WEB: www.rmklever.com
//  Mail: roymagne@rmklever.com
//
//  version 1.0, May 2010
//
//  MPL 1.1 licenced

interface

uses
  Windows, SysUtils, Classes, Controls, ExtCtrls, Graphics, Forms, Messages,
  ImgList, Math, Menus, Dialogs;
const
  CM_POPREFRESH = WM_USER + 1299; // Custom Message...
type
  PRGB24 = ^TRGB24;
  TRGB24 = packed record
    B: Byte;
    G: Byte;
    R: Byte;
  end;
  PLine24 = ^TLine24;
  TLine24 = array[0..0] of TRGB24;
  PRGBArray = ^TRGBArray;
  TRGBArray = array[0..0] of TRGB24;

  TrkGlassButton = class(TCustomControl)
  private
    ArrowX, ArrowY: Integer;
    GlyphX, GlyphY: Integer;
    TextX, TextY: Integer;
    FArrow: Boolean;
    FBmp: TBitmap;
    FBtnIdx: Integer;
    FButtonDown: Boolean;
    FCaption: string;
    FColor: TColor;
    FColorDown: TColor;
    FColorFrame: TColor;
    FDuoStyle: Boolean;
    FFont: TFont;
    //FGloss: Integer;
    FGlossy: Boolean;
    FGlossyLevel: Byte;
    FImages: TCustomImageList;
    FImageIndex: Integer;
    FNoClick: Boolean;
    FPopup: Boolean;
    FPopupAlignment: TPopupAlignment;
    FPopupMenu: TPopupMenu;
    FSingleBorder: Boolean;
    FOnPopup: TNotifyEvent;
    FOnMouseEnter: TNotifyEvent;
    FOnMouseLeave: TNotifyEvent;
    procedure InitPos;
    procedure CMMouseEnter(var Msg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    procedure CMIsToolControl(var Message: TMessage); message CM_ISTOOLCONTROL;
    procedure CMPopRefresh(var Message: TMessage); message CM_POPREFRESH;
    procedure WMERASEBKGND(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMWindowPosChanged(var Message: TWMWindowPosChanged); message WM_WINDOWPOSCHANGED;
    procedure PaintButton;
    procedure SetColor(const Value: TColor);
    procedure SetFrameColor(const Value: TColor);
    procedure SetColorDown(const Value: TColor);
    procedure SetFont(const Value: TFont);
    procedure SetCaption(const Value: string);
    procedure SetDuoStyle(const Value: Boolean);
    procedure SetArrow(const Value: Boolean);
    procedure SetImgIdx(const Value: Integer);
    procedure PaintAlphaArrow(ShowDown: Boolean);
    procedure PaintAlphaTxt(ShowDown, ShowArrow: Boolean);
    procedure SetSingleBrder(const Value: Boolean);
    procedure PaintAlphaRect(ABitmap: TBitmap; ARect: TRect; AColor: TColor; Alpha: Byte);
    procedure SetGlossy(const Value: Boolean);
    procedure SetGlossyLevel(const Value: Byte);
    { Private declarations }
  protected
    { Protected declarations }
    procedure Click; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure PaintWindow(DC: HDC); override;
    procedure Resize; override;
  public
    { Public declarations }
    FGloss: Integer;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ShowPopup;
  published
    { Published declarations }
    property Align;
    property Anchors;
    property Arrow: Boolean read FArrow write SetArrow default False;
    property AutoSize;
    property Caption: string read FCaption write SetCaption;
    property Color: TColor read FColor write SetColor default $00F4F4F4;
    property ColorDown: TColor read FColorDown write SetColorDown default clGray;
    property ColorFrame: TColor read FColorFrame write SetFrameColor default clSilver;
    property DuoStyle: Boolean read FDuoStyle write SetDuoStyle default False;
    property Glossy: Boolean read FGlossy write SetGlossy default False;
    property GlossyLevel: Byte read FGlossyLevel write SetGlossyLevel default 20;
    property ImageIdx: Integer read FImageIndex write SetImgIdx default -1;
    property DropDownAlignment: TPopupAlignment read FPopupAlignment write FPopupAlignment;
    property DropDownMenu: TPopupMenu read FPopupMenu write FPopupMenu;
    property Enabled;
    property Font: TFont read FFOnt write SetFont;
    property Images: TCustomImageList read FImages write FImages;
    property Left;
    property Top;
    property Width;
    property Height;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnPopup: TNotifyEvent read FOnPopup write FOnPopup;
    property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnKeyDown;
    property OnKeyUp;
    property OnResize;
    property SingleBorder: Boolean read FSingleBorder write SetSingleBrder default False;
  end;

procedure Register;

implementation

procedure DrawParentImage(Control: TControl; Dest: TCanvas);
var
  SaveIndex: Integer;
  DC: HDC;
  Position: TPoint;
begin
  with Control do
  begin
    if Parent = nil then
      Exit;
    DC := Dest.Handle;
    SaveIndex := SaveDC(DC);
{$IFDEF DFS_COMPILER_2}
    GetViewportOrgEx(DC, @Position);
{$ELSE}
    GetViewportOrgEx(DC, Position);
{$ENDIF}
    SetViewportOrgEx(DC, Position.X - Left, Position.Y - Top, nil);
    IntersectClipRect(DC, 0, 0, Parent.ClientWidth, Parent.ClientHeight);
    Parent.Perform(WM_ERASEBKGND, DC, 0);
    Parent.Perform(WM_PAINT, DC, 0);
    RestoreDC(DC, SaveIndex);
  end;
end;

procedure fxBoxBlur(Src: TBitmap; radius, rep: integer);
type
  PintRGB = ^TintRGB;
  TintRGB = packed record
    iB: Integer;
    iG: Integer;
    iR: Integer;
  end;
var
  j, divF, i, w, h, x, y, ny, tx, ty, prg, rad1: integer;
  ox, radm1, boxW, boxH: Integer;
  p: pRGB24;
  ptrS, ptrD, pv: integer;
  s0, s1: pRGB24;
  bool: boolean;
  lutRGB: array of TIntRGB;
  r, g, b: Byte;
begin
  if radius = 0 then
    Exit;
  divF := (radius * 2) + 1;
  w := Src.Width - 1;
  h := Src.Height - 1;
  SetLength(lutRGB, Max(w, h) + 1 + (radius * 2));
  s1 := Src.ScanLine[0];
  ptrD := integer(Src.ScanLine[1]) - integer(s1);
  rad1:= radius - 1;
  ny := Integer(s1);
  for y := 0 to h do
  begin
    for j := 1 to rep do
    begin
      for i := - radius to 0 do
      begin
        with pRGB24(ny)^, lutRGB[i + Radius] do
        begin
          iR := r + lutRGB[i + radius - 1].iR;
          iG := g + lutRGB[i + radius - 1].iG;
          iB := b + lutRGB[i + radius - 1].iB;
        end;
      end;
      for i := 1 to w do
      begin
        tx := i;
        with pRGB24(ny + tx * 3)^, lutRGB[i + Radius] do
        begin
          iR := r + lutRGB[i + radius - 1].iR;
          iG := g + lutRGB[i + radius - 1].iG;
          iB := b + lutRGB[i + radius - 1].iB;
        end;
      end;
      prg:= w * 3;
      for i := w + 1 to w + radius do
      begin
        with pRGB24(ny + prg)^, lutRGB[i + Radius] do
        begin
          iR := r + lutRGB[i + radius - 1].iR;
          iG := g + lutRGB[i + radius - 1].iG;
          iB := b + lutRGB[i + radius - 1].iB;
        end;
      end;
      ox := 0;
      for x := 0 to w do
      begin
        tx := x + radius;
        with pRGB24(ny + ox)^, lutRGB[tx + radius] do
        begin
          r := ((iR - lutRGB[tx - radius - 1].iR) div divF);
          g := ((iG - lutRGB[tx - radius - 1].iG) div divF);
          b := ((iB - lutRGB[tx - radius - 1].iB) div divF);
        end;
        inc(ox, 3);
      end;
    end;
    inc(ny, PtrD);
  end;
  ox := 0;
  for x := 0 to w do
  begin
    for j := 0 to rep do
    begin
      ny := Integer(s1);
      for i := -radius to 0 do
      begin
        with pRGB24(ny + x * 3)^, lutRGB[i + radius] do
        begin
          iR := r + lutRGB[i + radius - 1].iR;
          iG := g + lutRGB[i + radius - 1].iG;
          iB := b + lutRGB[i + radius - 1].iB;
        end;
      end;
      for i := 1 to h do
      begin
        inc(ny, PtrD);
        with pRGB24(ny + x * 3)^, lutRGB[i + radius] do
        begin
          iR := r + lutRGB[i + radius - 1].iR;
          iG := g + lutRGB[i + radius - 1].iG;
          iB := b + lutRGB[i + radius - 1].iB;
        end;
      end;
      for i := h + 1 to h + radius do
      begin
        with pRGB24(ny + x * 3)^, lutRGB[i + radius] do
        begin
          iR := r + lutRGB[i + radius - 1].iR;
          iG := g + lutRGB[i + radius - 1].iG;
          iB := b + lutRGB[i + radius - 1].iB;
        end;
      end;
      ny := Integer(s1);
      for y := 0 to h do
      begin
        ty := y + radius;
        with pRGB24(ny + ox)^, lutRGB[ty + radius] do
        begin
          r := ((iR - lutRGB[ty - radius - 1].iR) div divF);
          g := ((iG - lutRGB[ty - radius - 1].iG) div divF);
          b := ((iB - lutRGB[ty - radius - 1].iB) div divF);
        end;
        inc(ny, PtrD);
      end;
    end;
    inc(ox, 3);
  end;
  SetLength(lutRGB, 0);
  // Convert to greyscale
  s1 := Src.ScanLine[0];
  for y := 0 to h do
  begin
    for x := 0 to w do
    begin
      r:= PRGBArray(s1)[x].R;
      g:= PRGBArray(s1)[x].G;
      b:= PRGBArray(s1)[x].B;
      r:= Byte((r + g + b) div 3);
      PRGBArray(s1)[x].B := r;
      PRGBArray(s1)[x].G := r;
      PRGBArray(s1)[x].R := r;
    end;
    inc(Integer(s1), PtrD);
  end;
end;

function Blend(Color1, Color2: TColor; A: Byte): TColor;  inline;
var
  c1, c2: LongInt;
  r, g, b, v1, v2: byte;
begin
  A := Round(2.55 * A);
  c1 := ColorToRGB(Color1);
  c2 := ColorToRGB(Color2);
  v1 := Byte(c1);
  v2 := Byte(c2);
  r := Byte(A * (v1 - v2) shr 8 + v2);
  v1 := Byte(c1 shr 8);
  v2 := Byte(c2 shr 8);
  g := Byte(A * (v1 - v2) shr 8 + v2);
  v1 := Byte(c1 shr 16);
  v2 := Byte(c2 shr 16);
  b := Byte(A * (v1 - v2) shr 8 + v2);
  Result := (b shl 16) + (g shl 8) + r;
end;

procedure TrkGlassButton.PaintWindow(DC: HDC);
begin
  Canvas.Lock;
  try
    Canvas.Handle := DC;
    try
      FBmp.Canvas.Font.Assign(FFont);
      PaintButton;
    finally
      Canvas.Handle := 0;
    end;
  finally
    Canvas.Unlock;
  end;
end;

procedure PaintArrow(Canvas: TCanvas; x, y: Integer);
begin
  with Canvas do
  begin
    MoveTo(x + 3, y + 3);
    LineTo(x + 7, y + 3);
    LineTo(x + 5, Y + 5);
    LineTo(x + 3, y + 3);
    LineTo(x + 6, y + 5);
    MoveTo(x + 2, y + 2);
    LineTo(x + 9, y + 2);
  end
end;

procedure TrkGlassButton.PaintAlphaArrow(ShowDown: Boolean);
var
  row, slSize, x, y, i: Integer;
  rowSrc, slSizeSrc: Integer;
  slPnt, slPntSrc: PRGB24;
  Bmp: TBitmap;
begin
  Bmp := TBitmap.Create;
  try
    Bmp.PixelFormat:= pf24bit;
    Bmp.Width:= 24;
    Bmp.Height:= ClientHeight - 2;
    Bmp.Canvas.Brush.Color:= clWhite;
    Bmp.Canvas.FillRect(Bmp.Canvas.ClipRect);
    Bmp.Canvas.Pen.Color:= clBlack;
    PaintArrow(Bmp.Canvas, 9, ArrowY);
    fxBoxBlur(Bmp, 1, 3);
    if ShowDown then
      i:= 1
    else
      i:= 0;
    row:= Integer(FBmp.Scanline[i]);
    slSize:= Integer(FBmp.Scanline[i + 1]) - row;
    rowSrc:= Integer(Bmp.Scanline[0]);
    slSizeSrc:= Integer(Bmp.Scanline[1]) - rowSrc;
    row:= row  + (((FBmp.Width - 24) + i) * 3);
    for y := 0 to Bmp.Height - 1 do
    begin
      slPnt := pRGB24(row);
      slPntSrc := pRGB24(rowSrc);
      for x := 0 to Bmp.Width - 1 do
      begin
        slPnt.r := Byte(slPntSrc.R * (slPnt.R - 0) shr 8 + 0);
        slPnt.g := Byte(slPntSrc.R * (slPnt.G - 0) shr 8 + 0);
        slPnt.b := Byte(slPntSrc.R * (slPnt.B - 0) shr 8 + 0);
        inc(slPnt);
        Inc(slPntSrc);
      end;
      inc(row, slSize);
      inc(rowSrc, slSizeSrc);
    end;
    FBmp.Canvas.Pen.Color:= FBmp.Canvas.Font.Color;
    PaintArrow(FBmp.Canvas, ArrowX + i, ArrowY + i);
  finally
    Bmp.Free;
  end;
end;

procedure TrkGlassButton.PaintAlphaTxt(ShowDown, ShowArrow: Boolean);
var
  row, slSize, x, y, i: Integer;
  rowSrc, slSizeSrc: Integer;
  slPnt, slPntSrc: PRGB24;
  ts: TSize;
  Bmp, BmpAlpha: TBitmap;
  DC: HDC;
  Glyph: Boolean;
begin
  Glyph:= (FImageIndex <> -1) and (Assigned(FImages));
  BmpAlpha:= TBitmap.Create;
  try
    BmpAlpha.PixelFormat:= pf24bit;
    BmpAlpha.Width:= FBmp.Width - 2;
    BmpAlpha.Height:= FBmp.Height - 2;
    BmpAlpha.Canvas.Brush.Color:= clWhite;
    BmpAlpha.Canvas.FillRect(BmpAlpha.Canvas.ClipRect);
    if Glyph then
    begin
      Bmp := TBitmap.Create;
      try
        Bmp.Monochrome := True;
        FImages.GetBitmap(FImageIndex, bmp);
        BmpAlpha.Canvas.Brush.Color := clBlack;
        DC := BmpAlpha.Canvas.Handle;
        SetTextColor(DC, clBlack);
        SetBkColor(DC, clWhite);
        BitBlt(DC, GlyphX, GlyphY , Bmp.Width, Bmp.Height,
          Bmp.Canvas.Handle, 0, 0, SRCAND {ROP_DSPDxax});
      finally
        Bmp.Free;
      end;
    end;
    if FCaption <> '' then
    begin
      BmpAlpha.Canvas.Font.Assign(FFont);
      ts:= BmpAlpha.Canvas.TextExtent(FCaption);
      BmpAlpha.Canvas.Brush.Style:= bsClear;
      BmpAlpha.Canvas.Font.Color:= clBlack;
      BmpAlpha.Canvas.TextOut(TextX, TextY, FCaption);
    end;
    if ShowArrow then
    begin
      BmpAlpha.Canvas.Pen.Color:= clBlack;
      PaintArrow(BmpAlpha.Canvas, ArrowX, ArrowY);
    end;
    fxBoxBlur(BmpAlpha, 1, 3);
    if ShowDown then
      i:= 1
    else
      i:= 0;
    row:= Integer(FBmp.Scanline[i]);
    slSize:= Integer(FBmp.Scanline[i + 1]) - row;
    rowSrc:= Integer(BmpAlpha.Scanline[0]);
    slSizeSrc:= Integer(BmpAlpha.Scanline[1]) - rowSrc;
    row:= row  + (i * 3);
    for y := 0 to BmpAlpha.Height - 1 do
    begin
      slPnt := pRGB24(row);
      slPntSrc := pRGB24(rowSrc);
      for x := 0 to BmpAlpha.Width - 1 do
      begin
        slPnt.r := Byte(slPntSrc.R * (slPnt.R - 0) shr 8 + 0);
        slPnt.g := Byte(slPntSrc.R * (slPnt.G - 0) shr 8 + 0);
        slPnt.b := Byte(slPntSrc.R * (slPnt.B - 0) shr 8 + 0);
        inc(slPnt);
        Inc(slPntSrc);
      end;
      inc(row, slSize);
      inc(rowSrc, slSizeSrc);
    end;
    if Glyph then
    begin
      FImages.Draw(FBmp.Canvas, GlyphX + i, GlyphY + i, FImageIndex);
      FBmp.Canvas.TextOut(TextX + i, TextY + i, FCaption)
    end else
      FBmp.Canvas.TextOut(TextX + i, TextY + i, FCaption);
    if ShowArrow then
    begin
      FBmp.Canvas.Pen.Color:= FBmp.Canvas.Font.Color;
      PaintArrow(FBmp.Canvas, ArrowX + i, ArrowY + i);
    end;
  finally
    BmpAlpha.Free;
  end;
end;

procedure TrkGlassButton.PaintAlphaRect(ABitmap: TBitmap; ARect: TRect; AColor: TColor; Alpha: Byte);
var
  C: LongInt;
  i, row, slSize, x, y: Integer;
  Ra, Ga, Ba: array[0..255] of Byte;
  Col: TRGB24;
  slPnt: PRGB24;
  a, aStep, h: Integer;
begin
  C:= ColorToRGB(AColor);
  Col.B := (C shr 16) and $FF;
  Col.G := (C shr 8) and $FF;
  Col.R := C and $FF;
  for i := 0 to 255 do
  begin
    Ra[i] := Byte(Alpha * (Col.R - i) shr 8 + i);
    Ga[i] := Byte(Alpha * (Col.G - i) shr 8 + i);
    Ba[i] := Byte(Alpha * (Col.B - i) shr 8 + i);
  end;
  row:= Integer(ABitmap.Scanline[ARect.Top]);
  slSize:= Integer(ABitmap.Scanline[ARect.Top + 1]) - row;
  row:= row  + (ARect.Left * 3);
  h:= 0;
  if FGlossy then
  begin
    a:= 176;
    h:= (ARect.Bottom - ARect.Top) shr 1;
    aStep:= (a - Alpha) div h;
    for y := 0 to h -1 do
    begin
      slPnt := pRGB24(row);
      for x := ARect.Left to ARect.Right - 1 do
      begin
        slPnt.R := Byte(a * (Col.R - slPnt.R) shr 8 + slPnt.R);
        slPnt.G := Byte(a * (Col.G - slPnt.G) shr 8 + slPnt.G);
        slPnt.B := Byte(a * (Col.B - slPnt.B) shr 8 + slPnt.B);
        inc(slPnt)
      end;
      a:= a - aStep;
      inc(row, slSize);
    end;
  end;
  for y := h to ARect.Bottom - 1 do
  begin
    slPnt := pRGB24(row);
    for x := ARect.Left to ARect.Right - 1 do
    begin
      slPnt.R := Ra[slPnt.R];
      slPnt.G := Ga[slPnt.G];
      slPnt.B := Ba[slPnt.B];
      inc(slPnt)
    end;
    inc(row, slSize);
  end;
end;

procedure TrkGlassButton.PaintButton;
const
  TxtOp: Integer = DT_END_ELLIPSIS or DT_SINGLELINE or DT_NOPREFIX or
  DT_VCENTER;
var
  r, r1, r2: TRect;
  bool: Boolean;
  c1, c2, c3, c4: TColor;
begin
  InitPos;
  FBmp.Width := ClientWidth;
  FBmp.Height := ClientHeight;
  FBmp.Canvas.Brush.Style := bsClear;
  DrawParentImage(self, FBmp.Canvas);
  if (FBtnIdx <> -1) or (csDesigning in ComponentState) then
  begin
    r:= ClientRect;
    InflateRect(r, -1, -1);
    if not (csDesigning in ComponentState) then
      if FButtonDown then
      begin
        if FDuoStyle then
        begin
          r1:= r;
          r1.Right:= r1.Right - 18;
          r2:= r;
          r2.Left:= r2.Right - 17;
          if FBtnIdx = 0 then
          begin
            PaintAlphaRect(FBmp, r1, ColorDown, FGloss);
            PaintAlphaRect(FBmp, r2, Color, FGloss);
          end else
          begin
            PaintAlphaRect(FBmp, r2, ColorDown, FGloss);
            PaintAlphaRect(FBmp, r1, Color, FGloss);
          end;
        end else
          PaintAlphaRect(FBmp, r, ColorDown, FGloss);
      end else
        PaintAlphaRect(FBmp, r, Color, FGloss);
    r:= ClientRect;
    c1:= Blend(FBmp.Canvas.Pixels[r.Left, r.Bottom - 1], FBmp.Canvas.Pixels[r.Left, r.Top], 50);
    c2:= Blend(FBmp.Canvas.Pixels[r.Left, r.Top], FBmp.Canvas.Pixels[r.Right - 1, r.Top], 50);
    c3:= Blend(FBmp.Canvas.Pixels[r.Right - 1, r.Top], FBmp.Canvas.Pixels[r.Right -1, r. Bottom - 1], 50);
    c4:= Blend(FBmp.Canvas.Pixels[r.Left, r.Bottom - 1], FBmp.Canvas.Pixels[r.Right -1, r.Bottom -1], 50);
    c1:= Blend(c1, c2, 50);
    c2:= Blend(c3, c4, 50);
    c1:= Blend(c1, c2, 50);
    c1:= Blend(c1, FColorFrame, 50);
    FBmp.Canvas.Pen.Color:=  c1;
    FBmp.Canvas.Polyline([Point(r.Left, r.Top + 2), Point(r.Left + 2, r.Top),
      Point(r.Left + 2, r.Top), Point(r.Right - 3, r.Top), Point(r.Right - 1, r.Top + 2),
      Point(r.Right - 1, r.Bottom - 3), Point(r.Right - 3, r.Bottom - 1), Point(r.Left + 2, r.Bottom - 1),
      Point(r.Left, r.Bottom - 3), Point(r.Left, r.Top + 2)]);
    FBmp.Canvas.Pixels[r.Left, R. Top + 1] := Blend(FBmp.Canvas.Pixels[r.Left, R. Top + 1], c1, 50);
    FBmp.Canvas.Pixels[r.Left + 1, R. Top] := Blend(FBmp.Canvas.Pixels[r.Left + 1, R. Top], c1, 50);
    FBmp.Canvas.Pixels[r.Right - 1, R. Top + 1] := Blend(FBmp.Canvas.Pixels[r.Right -1, R. Top + 1], c1, 50);
    FBmp.Canvas.Pixels[r.Right - 2, R. Top] := Blend(FBmp.Canvas.Pixels[r.Right - 2, R. Top], c1, 50);
    FBmp.Canvas.Pixels[r.Left, R. Bottom - 2] := Blend(FBmp.Canvas.Pixels[r.Left, R. Bottom - 2], c1, 50);
    FBmp.Canvas.Pixels[r.Left + 1, R. Bottom - 1] := Blend(FBmp.Canvas.Pixels[r.Left + 1, R. Bottom -1], c1, 50);
    FBmp.Canvas.Pixels[r.Right - 1, R. Bottom - 2] := Blend(FBmp.Canvas.Pixels[r.Right -1, R. Bottom - 2], c1, 50);
    FBmp.Canvas.Pixels[r.Right - 2, R. Bottom - 1] := Blend(FBmp.Canvas.Pixels[r.Right - 2, R. Bottom - 1], c1, 50);
    if (not FSingleBorder) then
    begin
      c3:= Blend(c1, clWhite, 50);
      InflateRect(r, -1, -1);
      FBmp.Canvas.Pen.Color:=  c3;
      FBmp.Canvas.Polyline([Point(r.Left, r.Top + 1), Point(r.Left + 1, r.Top),
        Point(r.Left + 1, r.Top), Point(r.Right - 2, r.Top), Point(r.Right - 1, r.Top + 1),
        Point(r.Right - 1, r.Bottom - 2), Point(r.Right - 2, r.Bottom - 1), Point(r.Left + 1, r.Bottom - 1),
        Point(r.Left, r.Bottom - 2), Point(r.Left, r.Top + 1)]);
    end;
    r:= ClientRect;
    if FDuoStyle then
    begin
      FBmp.Canvas.MoveTo(r.Right - 19, r. Top + 1);
      FBmp.Canvas.LineTo(r.Right - 19, r.Bottom - 1);
      if FButtonDown then
      begin
        if FBtnIdx = 0 then
        begin
          FBmp.Canvas.Pen.Color:= Blend(c1, clBlack, 50);
          FBmp.Canvas.Polyline([Point(r.Left + 1, r.Bottom - 3), Point(r.Left + 1, r.Top + 2),
            Point(r.Left + 2, r.Top + 1), Point(r.Right - 19, r.Top + 1)]);
          FBmp.Canvas.Pen.Color:= Blend(c1, clWhite, 50);
          FBmp.Canvas.Polyline([Point(r.Right - 20, r.Top + 2), Point(r.Right - 20, r.Bottom - 3),
            Point(r.Right - 20, r.Bottom - 2), Point(r.Left + 1, r.Bottom - 2)]);
        end else
        begin
          FBmp.Canvas.Pen.Color:= Blend(c1, clBlack, 50);
          FBmp.Canvas.Polyline([Point(r.Right - 18, r.Bottom - 2), Point(r.Right - 18, r.Top + 1),
            Point(r.Right - 2, r.Top + 1), Point(r.Right - 2, r.Top + 1)]);
          FBmp.Canvas.Pen.Color:= Blend(c1, clWhite, 50);
          FBmp.Canvas.Polyline([Point(r.Right - 2, r.Top + 2), Point(r.Right - 2, r.Bottom - 3),
            Point(r.Right - 3, r.Bottom - 2), Point(r.Right - 18, r.Bottom - 2)]);
        end;
      end;
    end else
    begin
      if FButtonDown then
      begin
        FBmp.Canvas.Pen.Color:= Blend(c1, clBlack, 50);
        FBmp.Canvas.Polyline([Point(r.Left + 1, r.Bottom - 3), Point(r.Left + 1, r.Top + 2),
          Point(r.Left + 2, r.Top + 1), Point(r.Right - 2, r.Top + 1)]);
        FBmp.Canvas.Pen.Color:= Blend(c1, clWhite, 50);
        FBmp.Canvas.Polyline([Point(r.Right - 2, r.Top + 2), Point(r.Right - 2, r.Bottom - 3),
          Point(r.Right - 3, r.Bottom - 2), Point(r.Left + 1, r.Bottom - 2)]);
      end;
    end;
  end;
  if FDuoStyle then
  begin
    PaintAlphaArrow(FButtonDown and (FBtnIdx = 1));
    PaintAlphaTxt(FButtonDown and (FBtnIdx = 0), False);
  end else
    PaintAlphaTxt(FButtonDown, FArrow);
  BitBlt(Canvas.Handle, 0, 0, FBmp.Width, FBmp.Height, FBmp.Canvas.Handle, 0, 0,
    SRCCOPY);
end;

procedure TrkGlassButton.WMERASEBKGND(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure TrkGlassButton.WMPaint(var Message: TWMPaint);
begin
  PaintHandler(Message);
end;

procedure TrkGlassButton.Click;
begin
  if FNoClick then
    FNoClick := False
  else
    inherited;
end;

procedure TrkGlassButton.CMIsToolControl(var Message: TMessage);
begin
  Message.Result := 1;
end;

procedure TrkGlassButton.WMWindowPosChanged(var Message: TWMWindowPosChanged);
begin
  Invalidate;
  inherited;
  if not (csLoading in ComponentState) then
    Resize;
end;

procedure TrkGlassButton.SetArrow(const Value: Boolean);
begin
  FArrow := Value;
  Invalidate;
end;

procedure TrkGlassButton.SetCaption(const Value: string);
begin
  FCaption := Value;
  Invalidate;
end;

procedure TrkGlassButton.SetColor(const Value: TColor);
begin
  FColor := ColorToRGB(Value);
  Invalidate;
end;

procedure TrkGlassButton.SetColorDown(const Value: TColor);
begin
  FColorDown := Value;
  Invalidate;
end;

procedure TrkGlassButton.SetDuoStyle(const Value: Boolean);
begin
  FDuoStyle := Value;
  Invalidate;
end;

procedure TrkGlassButton.SetFont(const Value: TFont);
begin
  FFont.Assign(Value);
  Caption:= FCaption;
  Invalidate;
end;

procedure TrkGlassButton.SetFrameColor(const Value: TColor);
begin
  FColorFrame := ColorToRGB(Value);
  Invalidate;
end;

procedure TrkGlassButton.SetGlossy(const Value: Boolean);
begin
  FGlossy := Value;
  Invalidate;
end;

procedure TrkGlassButton.SetGlossyLevel(const Value: Byte);
var
  v: Integer;
begin
  v:= Value;
  if v < 0 then
    v:= 0;
  if v > 100 then
    v:= 100;
  FGlossyLevel := v;
  v:= MulDiv(FGlossyLevel, 64, 100);
  FGloss:= v;
  Invalidate;
end;

procedure TrkGlassButton.SetImgIdx(const Value: Integer);
begin
  FImageIndex := Value;
  Invalidate;
end;

procedure TrkGlassButton.SetSingleBrder(const Value: Boolean);
begin
  FSingleBorder := Value;
  Invalidate;
end;

procedure TrkGlassButton.ShowPopup;
var
  pt: TPoint;
begin
  if Assigned(FPopupMenu) then
  begin
    pt.X := 0;
    if FPopupAlignment = paCenter then
      pt.X := ClientWidth shr 1
    else
    if FPopupAlignment = paRight then
      pt.X := ClientWidth;
    pt.Y := ClientHeight;
    pt := ClientToScreen(pt);
    PaintButton;
    FPopup:= True;
    if Assigned(FOnPopup) then
      FOnPopup(Self);
    FPopupMenu.Alignment := FPopupAlignment;
    FPopupMenu.Popup(pt.X, pt.Y);
    PostMessage(Handle, CM_POPREFRESH, 0, 0);
  end;
end;

procedure TrkGlassButton.InitPos;
var
  ts: TSize;
begin
  ArrowX:= ClientWidth - 15;
  ArrowY:= (ClientHeight shr 1) - 4;
  GlyphX:= 4;
  GlyphY:= 0;
  if (FImageIndex <> -1) and Assigned(FImages) then
    GlyphY:= (ClientHeight - FImages.Height) shr 1;
  ts:= FBmp.Canvas.TextExtent(FCaption);
  TextX:= 4;
  if (FImageIndex <> -1) and Assigned(FImages) then
    TextX:= TextX + FImages.Height + 4;
  TextY:= (ClientHeight - ts.cy) shr 1;
end;

constructor TrkGlassButton.Create(AOwner: TComponent);
begin
  inherited;
  Color := clWhite;
  ColorFrame := clGray;
  ColorDown := clBlack;
  ControlStyle := [csClickEvents, csCaptureMouse, csSetCaption];
  if AOwner is TWinControl then
    Parent := TWinControl(AOwner);
  FBmp := TBitmap.Create;
  FBmp.PixelFormat := pf24Bit;
  FArrow:= False;
  FBtnIdx := -1;
  FButtonDown := False;
  FDuoStyle:= False;
  FFont := TFont.Create;
  FFont.Name := 'Tahoma';
  FFont.Color := clWhite;
  FFont.Size := 8;
  FGlossy:= False;
  GlossyLevel:= 37;
  FImageIndex:= -1;
  FNoClick:= False;
  FPopup:= False;
  FPopupAlignment := paLeft;
  FSingleBorder:= False;
  Height := 27;
  ParentFont := False;
  Width := 57;
  Caption:= 'Glass';
end;

destructor TrkGlassButton.Destroy;
begin
  FBmp.Free;
  FFont.Free;
  inherited Destroy;
end;

procedure TrkGlassButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  if (Button = mbLeft) then
  begin
    FButtonDown:= True;
    if x > ClientWidth - 20 then
    begin
      FBtnIdx:= 1;
      ShowPopup;
    end else
    begin
      FBtnIdx:= 0;
      PaintButton;
      if (not FDuoStyle) and FArrow then
        ShowPopup;
    end;
    PaintButton;
  end;
  inherited;
end;

procedure TrkGlassButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  FButtonDown := False;
  FBtnIdx := -1;
  FPopUp := False;
  Invalidate;
  inherited;
end;

procedure TrkGlassButton.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  idx: Integer;
begin
  if (PtInRect(ClientRect, Point(x, y))) and (not FButtonDown) then
  begin
    if x > ClientWidth - 20 then
      idx:= 1
    else
      idx:= 0;
    if idx <> FBtnIdx then
    begin
      FBtnIdx:= idx;
      Invalidate;
    end;
    FButtonDown:= (ssLeft In Shift);
  end;
  inherited;
end;

procedure TrkGlassButton.CMMouseEnter(var Msg: TMessage);
begin
  FButtonDown := False;
  FBtnIdx := -1;
  Invalidate;
  if Assigned(FOnMouseEnter) then
    FOnMouseEnter(Self);
  inherited;
end;

procedure TrkGlassButton.CMMouseLeave(var Msg: TMessage);
begin
  if FPopup then
    Exit;
  FButtonDown := False;
  FBtnIdx := -1;
  Invalidate;
  if Assigned(FOnMouseLeave) then
    FOnMouseLeave(Self);
  inherited;
end;

procedure TrkGlassButton.CMPopRefresh(var Message: TMessage);
begin
  if (csDestroying in ComponentState) then
    Exit;
  FBtnIdx:= -1;
  FButtonDown:= False;
  FPopUp := False;
  Invalidate;
  inherited;
end;

procedure TrkGlassButton.Resize;
begin
  Invalidate;
end;

procedure Register;
begin
  RegisterComponents('rmklever', [TrkGlassButton]);
end;

end.

